@@ -3,6 +3,7 @@ import sys
 import configuration
 import cli
 
+
 # Global configuration
 config = configuration.read_configuration()
 configuration.load_configuration(config)
@@ -14,5 +15,11 @@ def main():
   CLI = cli.HydratCmdln()
   sys.exit( CLI.main() )
 
+import classifier
+import dataset
+import corpora
+import langid
+import common.transform
+
 if __name__ == "__main__":
   main()
