@@ -22,6 +22,9 @@ def stratify(class_map):
   @type class_map: boolean class_map
   @rtype: boolean class_map
   """
+  return stratify_with_index(class_map)[0]
+
+def stratify_with_index(class_map):
   assert(class_map.dtype == 'bool')
   num_docs = class_map.shape[0]
 
@@ -45,7 +48,7 @@ def stratify(class_map):
   for doc_index, id in enumerate(strata_identifiers):
     strata_index                         = strata_indices[id]
     strata_map[doc_index, strata_index]  = True
-  return strata_map
+  return strata_map, strata_indices
 
 def allocate(strata_map, weights, probabilistic = False, rng = None):
   """
@@ -72,25 +75,30 @@ def allocate(strata_map, weights, probabilistic = False, rng = None):
 from hydrat.task.taskset import TaskSet
 from hydrat.task.task import InMemoryTask
 
-class Partitioning(object):
+class Partitioner(object):
   """ Represents a partitioning on a set of data """
-  def __init__(self, class_map, partitions, metadata):
+  def __init__(self, class_map, partitions, metadata, sequence=None):
+    # TODO: Do something with the sequence data
     # partitions is a 3-d array. instances X partitions X train/test(note order!)
     self.class_map = class_map
     self.parts = partitions
     self.metadata = dict(class_map.metadata)
     self.metadata.update(metadata)
 
+  def generate_metadata(self, feature_map, additional_metadata):
+    metadata = dict(self.metadata)
+    metadata.update(feature_map.metadata)
+    metadata.update(additional_metadata)
+    return metadata
+
   def __call__(self, feature_map, additional_metadata={}):
     """Transform a feature map into a TaskSet"""
     # Check the number of instances match
     assert feature_map.raw.shape[0] == self.parts.shape[0]
     # Check the feature map and class map are over the same dataset
-    assert feature_map.metadata['dataset_uuid'] == self.class_map.metadata['dataset_uuid']
+    assert feature_map.metadata['dataset'] == self.class_map.metadata['dataset']
     tasklist = []
-    metadata = dict(self.metadata)
-    metadata.update(feature_map.metadata)
-    metadata.update(additional_metadata)
+    metadata = self.generate_metadata(feature_map, additional_metadata)
     for i in range(self.parts.shape[1]):
       train_ids  = self.parts[:,i,0]
       test_ids   = self.parts[:,i,1]
@@ -116,6 +124,15 @@ class Sampler(object):
   def __call__(self, class_map):
     return self.sample(class_map)
     
+  @property
+  def metadata(self):
+    # Note that when this method is called is important: the RNG state
+    # may change.
+    metadata = {}
+    metadata['task_type'] = self.__class__.__name__
+    metadata['rng_state'] = self.rng.get_state()
+    return metadata
+
   def sample(self, class_map):
     """ Classes deriving sampler must implement this """
     raise NotImplementedError
@@ -124,22 +141,28 @@ def membership_vector(superset, subset):
   return numpy.fromiter((s in subset for s in superset), dtype=bool)
   
 class PresetSplit(Sampler):
-  def __init__(self, split, rng=None):
+  def __init__(self, split, metadata={}, rng=None):
     Sampler.__init__(self, rng)
+    self.additional_metadata = metadata
     self.split = split
 
   def sample(self, class_map):
-    metadata = dict()
-    metadata['task_type'] = 'preset_split'
-    metadata['rng_state'] = self.rng.get_state()
-    return Partitioning(class_map, self.split, metadata) 
+    metadata = self.metadata
+    metadata.update(self.additional_metadata)
+    return Partitioner(class_map, self.split, metadata) 
 
+from hydrat.common.decorators import deprecated
 class TrainTest(Sampler):
+  @deprecated
   def __init__(self, ratio=4, rng=None):
     Sampler.__init__(self, rng)
     self.ratio = ratio
 
   def sample(self, class_map):
+    # We must generate the metadata first as we need to know the RNG state
+    # when we got here, in order to re-create the partitioning.
+    metadata = self.metadata
+
     strata_map = stratify(class_map.raw)
     partition_proportions = numpy.array([self.ratio, 1])
     parts  = allocate( strata_map
@@ -149,17 +172,19 @@ class TrainTest(Sampler):
                      ) 
     # Build train/test by stacking to the correct shape
     parts = numpy.dstack((parts[:,0], parts[:,1])).swapaxes(0,1)
-    metadata = dict()
-    metadata['task_type'] = 'train_test'
-    metadata['rng_state'] = self.rng.get_state()
-    return Partitioning(class_map, parts, metadata) 
+    return Partitioner(class_map, parts, metadata) 
 
 class CrossValidate(Sampler):
+  @deprecated
   def __init__(self, folds=10, rng=None):
     Sampler.__init__(self, rng)
     self.folds = folds
 
   def sample(self, class_map):
+    # We must generate the metadata first as we need to know the RNG state
+    # when we got here, in order to re-create the partitioning.
+    metadata = self.metadata
+
     strata_map = stratify(class_map.raw)
     partition_proportions = numpy.array([1] * self.folds )
     folds = allocate( strata_map
@@ -170,7 +195,4 @@ class CrossValidate(Sampler):
     # We build train/test. Axis 0 is train, so it is the logical not of each fold, which represents
     # the test instances.
     folds = numpy.dstack((numpy.logical_not(folds), folds))
-    metadata = dict()
-    metadata['task_type'] = 'crossvalidate'
-    metadata['rng_state'] = self.rng.get_state()
-    return Partitioning(class_map, folds, metadata) 
+    return Partitioner(class_map, folds, metadata) 
